package io.syndesis.server.endpoint.v1.state;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.MoreObjects;
import com.google.common.collect.Lists;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.List;
import java.util.Objects;
import javax.annotation.CheckReturnValue;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;

/**
 * Immutable implementation of {@link ClientSideStateTest.Data}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableData.builder()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "ClientSideStateTest.Data"})
@Immutable
@CheckReturnValue
public final class ImmutableData implements ClientSideStateTest.Data {
  private final int integer;
  private final String string;

  private ImmutableData(int integer, String string) {
    this.integer = integer;
    this.string = string;
  }

  /**
   * @return The value of the {@code integer} attribute
   */
  @JsonProperty("integer")
  @Override
  public int getInteger() {
    return integer;
  }

  /**
   * @return The value of the {@code string} attribute
   */
  @JsonProperty("string")
  @Override
  public String getString() {
    return string;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ClientSideStateTest.Data#getInteger() integer} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for integer
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableData withInteger(int value) {
    if (this.integer == value) return this;
    return new ImmutableData(value, this.string);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ClientSideStateTest.Data#getString() string} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for string
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableData withString(String value) {
    if (this.string.equals(value)) return this;
    String newValue = Objects.requireNonNull(value, "string");
    return new ImmutableData(this.integer, newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableData} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableData
        && equalTo((ImmutableData) another);
  }

  private boolean equalTo(ImmutableData another) {
    return integer == another.integer
        && string.equals(another.string);
  }

  /**
   * Computes a hash code from attributes: {@code integer}, {@code string}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + integer;
    h += (h << 5) + string.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code Data} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("Data")
        .omitNullValues()
        .add("integer", integer)
        .add("string", string)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link ClientSideStateTest.Data} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Data instance
   */
  public static ImmutableData copyOf(ClientSideStateTest.Data instance) {
    if (instance instanceof ImmutableData) {
      return (ImmutableData) instance;
    }
    return ImmutableData.builder()
        .from(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ImmutableData ImmutableData}.
   * @return A new ImmutableData builder
   */
  public static ImmutableData.Builder builder() {
    return new ImmutableData.Builder();
  }

  /**
   * Builds instances of type {@link ImmutableData ImmutableData}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static final class Builder {
    private static final long INIT_BIT_INTEGER = 0x1L;
    private static final long INIT_BIT_STRING = 0x2L;
    private long initBits = 0x3L;

    private int integer;
    private @Nullable String string;

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code Data} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder from(ClientSideStateTest.Data instance) {
      Objects.requireNonNull(instance, "instance");
      integer(instance.getInteger());
      string(instance.getString());
      return this;
    }

    /**
     * Initializes the value for the {@link ClientSideStateTest.Data#getInteger() integer} attribute.
     * @param integer The value for integer 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("integer")
    public final Builder integer(int integer) {
      this.integer = integer;
      initBits &= ~INIT_BIT_INTEGER;
      return this;
    }

    /**
     * Initializes the value for the {@link ClientSideStateTest.Data#getString() string} attribute.
     * @param string The value for string 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("string")
    public final Builder string(String string) {
      this.string = Objects.requireNonNull(string, "string");
      initBits &= ~INIT_BIT_STRING;
      return this;
    }

    /**
     * Builds a new {@link ImmutableData ImmutableData}.
     * @return An immutable instance of Data
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableData build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableData(integer, string);
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = Lists.newArrayList();
      if ((initBits & INIT_BIT_INTEGER) != 0) attributes.add("integer");
      if ((initBits & INIT_BIT_STRING) != 0) attributes.add("string");
      return "Cannot build Data, some of required attributes are not set " + attributes;
    }
  }
}
